import lxml.etree as ET
import numpy as np


class Gait_Data( object ):

    """
    saves vhc-parameterization
    and start/end - kinematics 
    for gaits
    """


    def __init__( self ):

	pass



    #----------VHC_DATA_HANDLING----------------------
    
 
    def get_Element( self ) :

        """
        generating the root element of the XML data file storing the vhc parameters
        of the gait
        """
        
        et_root = ET.Element( "gait" ) 
   
        vhc_params = ET.SubElement( et_root, "vhc_parameters" )
        theta_space = ET.SubElement( et_root, "theta_space" )
        q_space = ET.SubElement( et_root, "q_space" )
        

        # ---------------------------vhc params------------------------------------ 
        P1, P2, P3 = self.vhc.get_parameterization()
        P_1, P_2, P_3 = self.vhc.get_parameter_strings()

       
        phi_1_parameters = ET.SubElement( vhc_params, "phi_1_parameters" )
        phi_2_parameters = ET.SubElement( vhc_params, "phi_2_parameters" )
        phi_3_parameters = ET.SubElement( vhc_params, "phi_3_parameters" )
       
        for i in range(0,len(P1)):
            p = ET.SubElement(phi_1_parameters, "vhc_parameter")
            p.set("p", P_1[i])
            p.text = str(P1[i])
        
        for i in range(0,len(P2)):
            p = ET.SubElement(phi_2_parameters, "vhc_parameter")
            p.set("p", P_2[i])
            p.text = str(P2[i])
        
        for i in range(0,len(P3)):
            p = ET.SubElement(phi_3_parameters, "vhc_parameter")
            p.set("p", P_3[i])
            p.text = str(P3[i])
        #----------------------------------------------------------------------------

        #------------------------theta-space-----------------------------------------
        
        theta_0 = self.theta_0
        theta_t = self.theta_t

        theta_0_d = self.theta_0_d
        theta_t_d = self.theta_t_d

        phi1, phi2, phi3 = self.vhc.get_func_vhc()
        
        k1, k2, k3 = self.theta_impact_map.get_factors() 
        
        Dphi1, Dphi2, Dphi3 = self.vhc.get_func_Dphi()
        
        theta_0_data = ET.SubElement( theta_space, "theta_0")
        theta_0_data.text = str(theta_0)
        
        theta_t_data = ET.SubElement( theta_space, "theta_t")
        theta_t_data.text = str(theta_t)
        
        theta_0_d_data = ET.SubElement( theta_space, "theta_0_d")
        theta_0_d_data.text = str(theta_0_d)
        
        theta_t_d_data = ET.SubElement( theta_space, "theta_t_d")
        theta_t_d_data.text = str(theta_t_d)

        
        
        impact_factors = ET.SubElement( theta_space, "theta_impact_factors")
        k1_data = ET.SubElement(impact_factors, "k1")
        k1_data.text = str(k1)
        
        k2_data = ET.SubElement(impact_factors, "k2")
        k2_data.text = str(k2)
        
        k3_data = ET.SubElement(impact_factors, "k3")
        k3_data.text = str(k3)
        
        #----------------------------------------------------------------------------

        #---------------------------q-config-----------------------------------------
        

        P1, P2, P3 = self.vhc.get_parameterization()
        P = self.vhc.get_full_parameterization_list()

        
        q1_0 = phi1(theta_0, P1)
        q2_0 = phi2(theta_0, P2)
        q3_0 = phi3(theta_0, P3)

        q1_t = phi1(theta_t, P1)
        q2_t = phi2(theta_t, P2)
        q3_t = phi3(theta_t, P3)
        
        q1_d_0 = Dphi1(theta_0, P1)*theta_0_d
        q2_d_0 = Dphi2(theta_0, P2)*theta_0_d
        q3_d_0 = Dphi3(theta_0, P3)*theta_0_d

        q1_d_t = Dphi1(theta_t, P1)*theta_t_d
        q2_d_t = Dphi2(theta_t, P2)*theta_t_d
        q3_d_t = Dphi3(theta_t, P3)*theta_t_d



        Q_0_data = ET.SubElement(q_space, "Q_0") 
        
        q1_0_data = ET.SubElement(Q_0_data, "q1_0")
        q1_0_data.text = str(q1_0)
        
        q2_0_data = ET.SubElement(Q_0_data, "q2_0")
        q2_0_data.text = str(q2_0)
        
        q3_0_data = ET.SubElement(Q_0_data, "q3_0")
        q3_0_data.text = str(q3_0)
        
        
        Q_d_0_data = ET.SubElement(q_space, "Q_d_0") 
        
        q1_d_0_data = ET.SubElement(Q_d_0_data, "q1_d_0")
        q1_d_0_data.text = str(q1_d_0)
        
        q2_d_0_data = ET.SubElement(Q_d_0_data, "q2_d_0")
        q2_d_0_data.text = str(q2_d_0)
        
        q3_d_0_data = ET.SubElement(Q_d_0_data, "q3_d_0")
        q3_d_0_data.text = str(q3_d_0)
       
        
        
        Q_t_data = ET.SubElement(q_space, "Q_t") 
        
        q1_t_data = ET.SubElement(Q_t_data, "q1_t")
        q1_t_data.text = str(q1_t)
        
        q2_t_data = ET.SubElement(Q_t_data, "q2_t")
        q2_t_data.text = str(q2_t)
        
        q3_t_data = ET.SubElement(Q_t_data, "q3_t")
        q3_t_data.text = str(q3_t)


        Q_d_t_data = ET.SubElement(q_space, "Q_d_t") 
        
        q1_d_t_data = ET.SubElement(Q_d_t_data, "q1_d_t")
        q1_d_t_data.text = str(q1_d_t)
        
        q2_d_t_data = ET.SubElement(Q_d_t_data, "q2_d_t")
        q2_d_t_data.text = str(q2_d_t)
        
        q3_d_t_data = ET.SubElement(Q_d_t_data, "q3_d_t")
        q3_d_t_data.text = str(q3_d_t)

        #-----------------------------------------------------------------------------


        return et_root
   
    def save_gait_to_XML( self, vhc, theta_impact_map, theta_0, theta_t,
		         theta_d_0, theta_d_t, xml_file_name ) : 
        
        """
        method: save_Model_XML( xml_file_name )
        saving the model data as an XML file
        """

        self.vhc = vhc
        self.theta_impact_map = theta_impact_map
	self.theta_0 = theta_0
	self.theta_t = theta_t
        self.theta_0_d = theta_d_0
        self.theta_t_d = theta_d_t
        tree = ET.ElementTree( self.get_Element() )
        
        tree.write( xml_file_name, pretty_print=True )


    def set_data_from_RootElement( self, root_element ) : 
        
        """
        data parsing from the root element of the XML file 
        containing the pysical parameters 
        ... very flexible for adding extra data to the data file format
        by adding subsequent parser functionality in the children classes 
        """
        
        parameters = root_element.find( "vhc_parameters" )
	theta_space = root_element.find( "theta_space" )


        theta_0 = float(theta_space.find( "theta_0" ).text)
        theta_t = float(theta_space.find( "theta_t" ).text)

        theta_d_0 = float(theta_space.find( "theta_0_d" ).text)
	theta_d_t = float(theta_space.find( "theta_t_d").text)
        
        P1 = []
        P2 = []
        P3 = []

        phi_1_parameters = parameters.find("phi_1_parameters")
        phi_2_parameters = parameters.find("phi_2_parameters")
        phi_3_parameters = parameters.find("phi_3_parameters")

        for p in phi_1_parameters.findall('vhc_parameter'):
            P1.append(float(p.text))
     
        for p in phi_2_parameters.findall('vhc_parameter'):
            P2.append(float(p.text))

        for p in phi_3_parameters.findall('vhc_parameter'):
            P3.append(float(p.text))
        
        
        self.vhc.parameterization(P1,P2,P3)

	self.theta_0 = theta_0
	self.theta_t = theta_t

	self.theta_d_0 = theta_d_0
	self.theta_d_t = theta_d_t


    def load_gait_from_XML( self, vhc, xml_file_name ) : 
        
        """
        method: load_Model_XML( xml_file_name )
        loading the model data from an XML file
        """

        self.vhc = vhc

        # getting the root element of the XML document 
        model = ET.parse( xml_file_name ).getroot()
        
        # passing it to the parser
        self.set_data_from_RootElement( model )


    def get_theta_0_t( self ):

	return self.theta_0, self.theta_t

    def get_theta_d_0_t( self ):
	
	return self.theta_d_0, self.theta_d_t


