import sys
sys.path.append("../theta_space_dynamics")
sys.path.append("../gait_search/")

from sympy import *
from theta_impact_map_v2 import Theta_Impact_Map
from gait_search_equations import Gait_Search_Equations
from vhc import VHC, VHC_Generator
from abg import Alpha_Beta_Gamma
from gait_data import Gait_Data
from motion_generator import Motion_Generator
import numpy as np
import matplotlib.pyplot as plt
from matplotlib import rc
np.set_printoptions(precision=6, linewidth = 200)


convention = "absolute"
model_xml_file = "../model_parameters_xml/CAD_params_ILeg.xml"
gait_file = "../thesis_report_gaits/rg_abs_2/rg_abs_2.xml"
plot_folder = '../thesis_report_gaits/rg_abs_2/'
#-----------------vhc init and load-----------------------------

phi_1 = VHC_Generator.polynomial(1,6)
phi_2 = VHC_Generator.polynomial(2,6)
phi_3 = VHC_Generator.polynomial(3,6)

vhc = VHC(phi_1, phi_2, phi_3)

gait_data = Gait_Data()
gait_data.load_gait_from_XML(vhc, gait_file)
#---------------------------------------------------------


# getting start/end theta and theta_dot 
theta_0, theta_t = gait_data.get_theta_0_t()
theta_d_0, theta_d_t = gait_data.get_theta_d_0_t()

print "angle_convention: ", convention
print "physical model parameters:", model_xml_file
print "theta_0 =", theta_0
print "theta_t =", theta_t
print "theta_0_d=", theta_d_0
print "theta_t_d =", theta_d_t
print "\n\n"

# ----theta space init---------
abg = Alpha_Beta_Gamma( convention, model_xml_file, vhc)
abg.create_abg0_sim()
abg.create_tau_sim()
theta_impact_map = Theta_Impact_Map( convention, model_xml_file, vhc, theta_0, theta_t )
gse = Gait_Search_Equations( convention, abg, vhc, theta_impact_map)



#============= GAIT ANALYSIS AND PLOTTING======================

# motion generator
mg = Motion_Generator(abg, vhc)

##init conditions
t = np.linspace(0.0, 3.0, 3000)
theta_theta_d_0 = np.array([theta_0, theta_d_0])

# sim
print "sim..."
theta_traj, T = mg.abg0_sim_until_value(theta_theta_d_0, 1.0, t)

# generate q-traj and torques
Q_Qd = mg.get_Q_Qd()
tau = mg.get_torques()




#plotting
#------------------------------


rc('font',**{'family':'normal','size':22})
rc('text', usetex=True)


# plotting alpha
plt.figure(1)
plt.xticks(fontsize=14)
plt.yticks(fontsize=14)
plt.ylabel(r'$\alpha(\theta)$')
plt.xlabel(r'$t$')
plt.plot(theta_traj[:,0], abg.alpha0_sim(theta_traj[:,0]))
plt.tight_layout()
plt.draw()
plt.savefig(plot_folder+'alpha.eps', format='eps', dpi=1000)

# plotting theta,theta_dot -phaseportrait
plt.figure(2)
plt.xticks(fontsize=14)
plt.yticks(fontsize=14)
plt.xlabel(r'$\mathbf{\theta}$',fontsize=30)
plt.ylabel(r'$\mathbf{\dot{\theta}}$',fontsize=30)
plt.plot( theta_traj[:,0], theta_traj[:,1] )
plt.tight_layout()
plt.draw()
plt.savefig(plot_folder+'theta_pp.eps', format='eps', dpi=1000)

#plot q(t)
plt.figure(3)
plt.xticks(fontsize=14)
plt.yticks(fontsize=14)
plt.ylabel(r'$\mathbf{q}(t)$',fontsize=30)
plt.xlabel('$\mathbf{t}$',fontsize=30)
plt.plot(T, Q_Qd[:,0], 'b', label=r'$q_1$')
plt.plot(T, Q_Qd[:,1], 'g', label=r'$q_2$')
plt.plot(T, Q_Qd[:,2], 'r', label=r'$q_3$')
plt.legend(loc=2)
plt.tight_layout()
plt.draw()
plt.savefig(plot_folder+'q_time.eps', format='eps', dpi=1000)

#plot q1, q2, q3 - phase_portrait  
plt.figure(4)
plt.xticks(fontsize=14)
plt.yticks(fontsize=14)
plt.xlabel(r'$\mathbf{q}$',fontsize=30)
plt.ylabel(r'$\dot{\mathbf{q}}$',fontsize=30)
plt.plot(Q_Qd[:,0], Q_Qd[:,3], 'b', label=r'$q_1$')
plt.plot(Q_Qd[:,1], Q_Qd[:,4], 'g', label=r'$q_2$')
plt.plot(Q_Qd[:,2], Q_Qd[:,5], 'r', label=r'$q_3$')
plt.plot( [ Q_Qd[-1,0], Q_Qd[0,2] ], [ Q_Qd[-1,3], Q_Qd[0,5] ], 'k--' )
plt.plot( [ Q_Qd[-1,2], Q_Qd[0,0] ], [ Q_Qd[-1,5], Q_Qd[0,3] ], 'k--' )
plt.plot( [ Q_Qd[-1,1], Q_Qd[0,1] ], [ Q_Qd[-1,4], Q_Qd[0,4] ], 'k--' )
plt.legend(loc=3)
plt.tight_layout()
plt.draw()
plt.savefig(plot_folder+'q_pp.eps', format='eps', dpi=1000)

# plot torques
plt.figure(5)
plt.xticks(fontsize=14)
plt.yticks(fontsize=14)
plt.xlabel(r'$\mathbf{t}$')
plt.ylabel(r'$\mathbf{u}\;\;[Nm]$')
plt.plot(T, tau[:,0], 'b', label=r'$u_2$' )
plt.plot(T, tau[:,1], 'r', label=r'$u_3$' )
plt.legend(loc=3)
plt.tight_layout()
plt.draw()
plt.savefig(plot_folder+'single_step_torque.eps', format='eps', dpi=1000)



