import sys
sys.path.append("../theta_space_dynamics")
sys.path.append("../gait_search/")

from sympy import *
from theta_impact_map_v2 import Theta_Impact_Map
from gait_search_equations import Gait_Search_Equations
from vhc import VHC, VHC_Generator
from abg import Alpha_Beta_Gamma
from gait_data import Gait_Data
from motion_generator import Motion_Generator
import numpy as np
import matplotlib.pyplot as plt
from matplotlib import rc
rc('font',**{'family':'normal','size':22})
rc('text', usetex=True)

np.set_printoptions(precision=6, linewidth = 200)


convention = "absolute"
model_xml_file = "../model_parameters_xml/CAD_params_ILeg.xml"
gait_file = "../thesis_report_gaits/rg_abs_2/rg_abs_2.xml"
plot_folder = '../thesis_report_gaits/rg_abs_2/'
#-----------------vhc init and load-----------------------------

phi_1 = VHC_Generator.polynomial(1,6)
phi_2 = VHC_Generator.polynomial(2,6)
phi_3 = VHC_Generator.polynomial(3,6)

vhc = VHC(phi_1, phi_2, phi_3)

gait_data = Gait_Data()
gait_data.load_gait_from_XML(vhc, gait_file)
#---------------------------------------------------------


# getting start/end theta and theta_dot 
theta_0, theta_t = gait_data.get_theta_0_t()
theta_d_0, theta_d_t = gait_data.get_theta_d_0_t()

print "angle_convention: ", convention
print "physical model parameters:", model_xml_file
print "theta_0 =", theta_0
print "theta_t =", theta_t
print "theta_0_d=", theta_d_0
print "theta_t_d =", theta_d_t
print "\n\n"

# ----theta space init---------
abg = Alpha_Beta_Gamma( convention, model_xml_file, vhc)
abg.create_abg0_sim()
abg.create_tau_sim()
theta_impact_map = Theta_Impact_Map( convention, model_xml_file, vhc, theta_0, theta_t )
gse = Gait_Search_Equations( convention, abg, vhc, theta_impact_map)

#============= GAIT SIMULATION AND PLOTTING======================

# motion generator
mg = Motion_Generator(abg, vhc)

##init conditions
t = np.linspace(0.0, 2.0, 1000)
theta_theta_d_0 = np.array([theta_0, theta_d_0])
step = 0
number_of_steps = 3
first = True


# sim
print "sim..."
while step < number_of_steps:

    if first:

        gait_theta_traj, gait_T = mg.abg0_sim_until_value(theta_theta_d_0, 1.0, t)
        
        # generate q-traj and torques
        gait_Q_Qd = mg.get_Q_Qd()
        gait_tau = mg.get_torques()

        first = False
        
        # plot q(t)
        plt.figure(1)
        plt.plot(gait_T, gait_Q_Qd[:,0], 'b', label=r'$q_1$')
        plt.plot(gait_T, gait_Q_Qd[:,1], 'g', label=r'$q_2$')
        plt.plot(gait_T, gait_Q_Qd[:,2], 'r', label=r'$q_3$')
        plt.axvline(x=gait_T[-1],color='k', ls = '--')
        #plt.draw()

        ## plot torques
        plt.figure(2)
        plt.plot(gait_T, gait_tau[:,0], 'b',label=r'$u_2$')
        plt.plot(gait_T, gait_tau[:,1], 'r',label=r'$u_3$')
        plt.axvline(x=gait_T[-1],color='k', ls = '--')
        #plt.draw()
        
        theta_d_0_arr, K = theta_impact_map.impact(gait_theta_traj[-1,1])
        theta_theta_d_0 = np.array([float(gait_theta_traj[0,0]),float(theta_d_0_arr[1])])
    
    else:

        step_theta_traj, step_T = mg.abg0_sim_until_value(theta_theta_d_0, 1.0, t)

        # generate q-traj and torques
        step_Q_Qd = mg.get_Q_Qd()
        step_tau = mg.get_torques()

        #plot q(t) 
        plt.figure(1)
        plt.plot(step_T + step*step_T[-1], step_Q_Qd[:,0], 'b')
        plt.plot(step_T + step*step_T[-1], step_Q_Qd[:,1], 'g')
        plt.plot(step_T + step*step_T[-1], step_Q_Qd[:,2], 'r')
        plt.axvline(x=step*step_T[-1],color='k', ls = '--')

        ## plot torques
        plt.figure(2)
        plt.plot(step_T + step*step_T[-1], step_tau[:,0], 'b')
        plt.plot(step_T + step*step_T[-1], step_tau[:,1], 'r')
        plt.axvline(x=step*step_T[-1],color='k', ls = '--')
        
        gait_theta_traj = np.concatenate((gait_theta_traj,step_theta_traj),axis=0)
        gait_T = np.concatenate((gait_T,step_T + step*step_T[-1]),axis=0)
        
        gait_Q_Qd = np.concatenate((gait_Q_Qd,step_Q_Qd),axis=0)
        gait_tau = np.concatenate((gait_tau,step_tau),axis=0)
        
        # update theta-trajectory
        theta_d_0_arr, K = theta_impact_map.impact(step_theta_traj[-1,1])
        theta_theta_d_0 = np.array([float(step_theta_traj[0,0]),float(theta_d_0_arr[1])])
    
    
    step += 1

plt.figure(1)
plt.axvline(x=step*step_T[-1],color='k', ls = '--')
plt.figure(2)
plt.axvline(x=step*step_T[-1],color='k', ls = '--')

#plot q1, q2, q3 
plt.figure(1)
plt.xticks(fontsize=14)
plt.yticks(fontsize=14)
plt.ylabel(r'$\mathbf{q}(t)$',fontsize=30)
plt.xlabel('$\mathbf{t}$',fontsize=30)
plt.legend(loc=1)
plt.tight_layout()
plt.draw()
plt.savefig(plot_folder+'cyclic_q_time.eps', format='eps', dpi=1000)

# plot torques
plt.figure(2)
plt.xticks(fontsize=14)
plt.yticks(fontsize=14)
plt.xlabel(r'$\mathbf{t}$',fontsize=30)
plt.ylabel(r'$\mathbf{u}\;\;[Nm]$',fontsize=30)
plt.legend(loc=1)
plt.tight_layout()
plt.draw()
plt.savefig(plot_folder+'cyclic_torque.eps', format='eps', dpi=1000)




