import sys 
sys.path.append("../theta_space_dynamics/")
sys.path.append("../kinematics/")

from sympy import *
from theta_impact_map_v2 import Theta_Impact_Map
from theta_space_kin import Theta_Space_Kinematics
from vhc import VHC, VHC_Generator
from abg import Alpha_Beta_Gamma
from gait_search_equations import Gait_Search_Equations
from gait_data import Gait_Data
import numpy as np
import scipy.optimize
import math
import datetime
import time
np.set_printoptions(precision=9, linewidth = 200)

#angle_convention
convention = "relative"

#physical paramater model xml-file
model_parameters_xml = "../model_parameters_xml/CAD_params_ILeg.xml" 

# write to file
out_dir = "gaits_xml/"
out_file_name = "realizable_gait"


# ============defining VHCs===================================
t = symbols('t')
theta = Function('theta')(t)

phi_1 = VHC_Generator.polynomial(1,6) 
phi_2 = VHC_Generator.polynomial(2,6)
phi_3 = VHC_Generator.polynomial(3,6)
# =============================================================

#===================INIT=============================

# theta start/end
theta_0 = 0.0
theta_t = 1.0

torque_theta_vec = np.linspace(theta_0, theta_t, 200)
vel_theta_vec = np.linspace(theta_0, theta_t, 20)

# init config at t = 0
#q_1_max = 0.25 
#q_1_min = 0.05
#q_2_max = 0.2
#q_2_min = -0.2

q_1_max = 2.0 
q_1_min = 1.6
q_2_max = 0.0
q_2_min = - 1.0

k_min = 0.5

print "q_1_0_max: ", q_1_max
print "q_1_0_min: ", q_1_min
print "q_2_max: ", q_2_max
print "q_2_min: ", q_2_min
print "\n"


#=========================================================

#================== setting up theta-space dynamics ===========
vhc = VHC(phi_1, phi_2, phi_3)

theta_impact_map = Theta_Impact_Map( convention, model_parameters_xml, vhc, theta_0, theta_t )
theta_impact_map.derive_factor_grad()

kin = Theta_Space_Kinematics(convention, model_parameters_xml, vhc)

abg = Alpha_Beta_Gamma(convention, model_parameters_xml, vhc)

gse = Gait_Search_Equations( convention, abg, vhc, theta_impact_map )
gse.derive_theta_impact_obj_grad()
gse.derive_gait_symmetry_con_grad(theta_0, theta_t)
gse.derive_q1_q3_init_symmetry_con_grad(theta_0)
gse.derive_q_init_grad(theta_0)

gait_data = Gait_Data()

print "\nINIT DONE"

#============================================================


#================optimalization wrappers==========================

def theta_impact_con(P):
    
    vhc.alt_parameterization(P)
    return gse.eval_theta_impact_obj_func()

def theta_impact_con_grad(P):
     
    vhc.alt_parameterization(P) 
    return gse.eval_theta_impact_obj_grad()

def symmetry_con(P):
     
    vhc.alt_parameterization(P)
    return gse.eval_gait_symmetry_con_lhs(theta_0, theta_t) 

def symmetry_con_grad(P):
     
    vhc.alt_parameterization(P)
    return gse.eval_gait_symmetry_con_grad(P) 

def q_1_init_con_max(P):
    
    vhc.alt_parameterization(P)
    return gse.q_1_init_con_lhs_max(theta_0, q_1_max)

def q_1_init_con_max_grad(P):

    vhc.alt_parameterization(P)
    return gse.q1_max_grad(P)

def q_1_init_con_min(P):
    
    vhc.alt_parameterization(P)
    return gse.q_1_init_con_lhs_min(theta_0, q_1_min)

def q_1_init_con_min_grad(P):

    vhc.alt_parameterization(P)
    return gse.q1_min_grad(P)

def q_2_init_con_max(P):

    vhc.alt_parameterization(P)
    return gse.q_2_init_con_lhs_max(theta_0, q_2_max)

def q_2_init_con_max_grad(P):

    vhc.alt_parameterization(P)
    return gse.q2_max_grad(P)

def q_2_init_con_min(P):

    vhc.alt_parameterization(P)
    return gse.q_2_init_con_lhs_min(theta_0, q_2_min)

def q_2_init_con_min_grad(P):

    vhc.alt_parameterization(P)
    return gse.q2_min_grad(P)

def q1_q3_init_symmetry_con(P):

    vhc.alt_parameterization(P)
    return gse.q1_q3_init_symmetry_con_lhs(theta_0)

def q1_q3_init_symmetry_con_grad(P):

    vhc.alt_parameterization(P)
    return gse.q1_q3_init_grad(P)

def k_1_min(P):

    vhc.alt_parameterization(P)
    return theta_impact_map.k1(P) - k_min

def k_2_min(P):

    vhc.alt_parameterization(P)
    return theta_impact_map.k2(P) - k_min

def k_3_min(P):

    vhc.alt_parameterization(P)
    return theta_impact_map.k3(P) - k_min

def k_1_grad(P):

    vhc.alt_parameterization(P)
    return theta_impact_map.k1_grad(P)

def k_2_grad(P):

    vhc.alt_parameterization(P)
    return theta_impact_map.k2_grad(P)

def k_3_grad(P):

    vhc.alt_parameterization(P)
    return theta_impact_map.k3_grad(P)


def swing_leg_lift_off(P):

    vhc.alt_parameterization(P)
    return kin.Dp3f(theta_0, P)[1] 

def swing_leg_downstroke(P):

    vhc.alt_parameterization(P)
    return - kin.Dp3f(theta_t, P)[1]



def min_theta_d_sq_con(P):

    vhc.alt_parameterization(P)

    rho1_arr, rho2_arr = gse.fast_rho1_rho2_arr( vel_theta_vec )
    
    lhs = gse.min_theta_d_sq( rho1_arr, rho2_arr )
    
    return lhs

def max_torque(P):

    vhc.alt_parameterization(P)

    lhs = gse.eval_max_torque_con(vel_theta_vec, 1.0)
    
    return lhs

def torque_continuity(P):
    
    vhc.alt_parameterization(P)
    

    rho1_arr, rho2_arr = gse.fast_rho1_rho2_arr( torque_theta_vec )
    theta_d_sq = gse.fast_theta_d_sq(rho1_arr, rho2_arr)

    theta_d_0_sq = theta_d_sq[0]
    theta_d_t_sq = theta_d_sq[-1]
   
    lhs = gse.eval_torque_con_lhs(theta_0, theta_t, theta_d_0_sq, theta_d_t_sq)
    
    return lhs

def max_alpha(P):

    vhc.alt_parameterization(P)

    return gse.eval_max_alpha_con(vel_theta_vec, 0.5)


#=========================================================================


#SLSQP set-up
#------------------------------------------------------------------------------------------------


con_1 = {'type':'eq', 'fun':symmetry_con, 'jac':symmetry_con_grad}
con_2 = {'type':'eq', 'fun':q1_q3_init_symmetry_con, 'jac':q1_q3_init_symmetry_con_grad}

con_3 = {'type':'ineq', 'fun':q_1_init_con_max, 'jac':q_1_init_con_max_grad}
con_4 = {'type':'ineq', 'fun':q_1_init_con_min, 'jac':q_1_init_con_min_grad}
con_5 = {'type':'ineq', 'fun':q_2_init_con_max, 'jac':q_2_init_con_max_grad}
con_6 = {'type':'ineq', 'fun':q_2_init_con_min, 'jac':q_2_init_con_min_grad}

con_7 = {'type':'ineq', 'fun':k_1_min, 'jac':k_1_grad} 
con_8 = {'type':'ineq', 'fun':k_2_min, 'jac':k_2_grad} 
con_9 = {'type':'ineq', 'fun':k_3_min, 'jac':k_3_grad}

con_10 = {'type':'ineq', 'fun':swing_leg_lift_off}
con_11 = {'type':'ineq', 'fun':swing_leg_downstroke}

con_12 = {'type':'ineq', 'fun':min_theta_d_sq_con}
con_13 = {'type':'ineq', 'fun':max_alpha}

con_14 = {'type':'ineq', 'fun':max_torque} 

con_15 = {'type':'eq', 'fun':theta_impact_con, 'jac':theta_impact_con_grad}



# vhc load
gait_file = "gaits_xml/some_gait.xml"
gait_data.load_gait_from_XML(vhc,gait_file)

x0 = vhc.get_full_parameterization_list()

# optimalization options
opt_options = {'disp': True, 'maxiter': 1000, 'ftol': 1e-12}

#------------------------------------------------------------------------------------------------


print "SLSQP searching..."
print"-----------------------------------------------------------------------------------------"


print "\nx0: ", x0
print "\n"

res = scipy.optimize.minimize( 
        
        torque_continuity, x0, args=(), method='SLSQP', jac=None, 
        options=opt_options, bounds=None, 
        constraints = ( con_1, con_2, con_3, con_4, 
                        con_5, con_6, con_7, con_8, 
                        con_9, con_10, con_11, con_12, 
                        con_13, con_14, con_15 ) 
        )

print "\n"
print res
print "\n"

P = res.x

print "vhc_params: "
print P
print "\n"


fine_theta_vec = np.linspace(theta_0, theta_t, 1000)
rho1_arr, rho2_arr = gse.fast_rho1_rho2_arr(fine_theta_vec)
f_theta_d_0_sq, f_theta_d_t_sq = gse.fast_theta_d_0_t_sq( rho1_arr, rho2_arr )
f_theta_d_sq = gse.fast_theta_d_sq( rho1_arr, rho2_arr )


s_theta_d_0_sq, s_theta_d_t_sq = gse.slow_theta_d_0_t_sq(theta_0,theta_t,P)

print "f_theta_d_0_sq: ", f_theta_d_0_sq
print "f_theta_d_t_sq: ", f_theta_d_t_sq
print "min(f_theta_d_sq): ", np.amin(f_theta_d_sq)


print "\ns_theta_d_0_sq: ", s_theta_d_0_sq
print "s_theta_d_t_sq: ", s_theta_d_t_sq

theta_d_0 = np.sqrt(f_theta_d_0_sq)
theta_d_t = np.sqrt(f_theta_d_t_sq)           

print "f_theta_d_0: ", theta_d_0
print "f_theta_d_t: ", theta_d_t

print "\ntorque-constraints: "

print "max_torque: ", max_torque(P)
print "torque_continuity: ", torque_continuity(P)




# saving gait to xml-file
#---------------------------------------------
if res.success:
    now = datetime.datetime.now()
    the_date = str(now.month) + "_" + str(now.day)
    the_time = str(now.hour) + "_" + str(now.minute) + "_" + str(now.second) 
    out_file = out_dir + out_file_name + "_" + the_date + "_" + the_time + ".xml"



    gait_data.save_gait_to_XML(vhc,theta_impact_map, theta_0, theta_t, 
                            theta_d_0, theta_d_t, out_file)

    print "\n\nWROTE TO FILE: ", out_file
    
