from biped_data import Biped_Data
from sympy import *
import numpy as np
import lxml.etree as ET



class Theta_Space_Kinematics( Biped_Data ):
    
    
    def __init__( self, convention, model_xml, vhc ):


        self.t = symbols('t')
        self.theta = Function('theta')('t')
        self.theta_d = diff(self.theta,self.t)

        self.vhc = vhc

        super( Theta_Space_Kinematics, self ).__init__()
        
        self.load_Model_XML(model_xml)
         
        if convention == "absolute":
            self.absolute_angles()
        
        elif convention == "relative":
            self.relative_angles()
        
        else:
            print "unvalid angle convention given to kinematics"
            self.absolute_angles()

        self.lambdify_kinematics()

    
    
        
    def absolute_angles( self ):
        
        """
        Derivation of kinematics in relative angles        
        """

        t = self.t
        theta = self.theta
        theta_d = self.theta_d

        phi_1, phi_2, phi_3 = self.vhc.get_sym_vhcs()

        #--------- Symbolic Definitions -----------------------

        #define names of physical parameters as symbols
        l_1 = symbols('l_1')
        l_2 = symbols('l_2')
        l_3 = symbols('l_3')

        L_1 = symbols('L_1')
        L_2 = symbols('L_2')
        L_3 = symbols('L_3')

        m_1 = symbols('m_1')
        m_2 = symbols('m_2')
        m_3 = symbols('m_3')

        J_1 = symbols('J_1')
        J_2 = symbols('J_2')
        J_3 = symbols('J_3')

        g = symbols('g')


        #define generalized coordinates as functions of t
        #"Function" is a method from sympy module defining q_i to be functions of t
        t=symbols('t')
        q_1=Function('q_1')(t)
        q_2=Function('q_2')(t)
        q_3=Function('q_3')(t)

        #-----------------------------------------------------


        # Positions
        
        p_1=Matrix([
            [-l_1*sin(q_1)],
            [l_1*cos(q_1)]
            ])
        
        p_2=Matrix([
            [-L_1*sin(q_1)-l_2*sin(q_2)],
            [L_1*cos(q_1)+l_2*cos(q_2)]
            ])
        
        p_3=Matrix([
            [-L_1*sin(q_1)+l_3*sin(q_3)],
            [L_1*cos(q_1)-l_3*cos(q_3)]
            ])
        
        p_3f=Matrix([
            [-L_1*sin(q_1)+L_3*sin(q_3)],
            [L_1*cos(q_1)-L_3*cos(q_3)]
            ])

        # ---------------------------------------------------
        
        #substituting in vhcs
        vhc_subs = [ (q_1,phi_1),(q_2,phi_2),(q_3,phi_3) ]
        p_1, p_2, p_3, p_3f = p_1.subs(vhc_subs), p_2.subs(vhc_subs), p_3.subs(vhc_subs), p_3f.subs(vhc_subs)

        #Linear Velocities
        v_1=p_1.diff(t)
        v_2=p_2.diff(t)
        v_3=p_3.diff(t)
        v_3f=p_3f.diff(t)
        
        # derivative of swing-leg foot with repsect to theta
        Dp_3f = p_3f.diff(theta)


        # setting symbolic kin as instance var
        self.p_1, self.p_2, self.p_3, self.p_3f = p_1, p_2, p_3, p_3f
        self.Dp_3f = Dp_3f
        self.v_1, self.v_2, self.v_3, self.v_3f = v_1, v_2, v_3, v_3f
        
        #------------------------------------------------------

    
    
   
   
    def relative_angles( self ):
        
        """
        Derivation of kinematics in relative angles        
        """

        t = self.t
        theta = self.theta
        theta_d = self.theta_d

        phi_1, phi_2, phi_3 = self.vhc.get_sym_vhcs()

        #--------- Symbolic Definitions -----------------------

        #define names of physical parameters as symbols
        l_1 = symbols('l_1')
        l_2 = symbols('l_2')
        l_3 = symbols('l_3')

        L_1 = symbols('L_1')
        L_2 = symbols('L_2')
        L_3 = symbols('L_3')

        m_1 = symbols('m_1')
        m_2 = symbols('m_2')
        m_3 = symbols('m_3')

        J_1 = symbols('J_1')
        J_2 = symbols('J_2')
        J_3 = symbols('J_3')

        g = symbols('g')


        #define generalized coordinates as functions of t
        #"Function" is a method from sympy module defining q_i to be functions of t
        t=symbols('t')
        q_1=Function('q_1')(t)
        q_2=Function('q_2')(t)
        q_3=Function('q_3')(t)

        #-----------------------------------------------------


        #Position 
        p_1=Matrix([
            [l_1*cos(q_1)],
            [l_1*sin(q_1)]
            ])
        p_2=Matrix([
            [L_1*cos(q_1)+l_2*cos(q_1+q_2)],
            [L_1*sin(q_1)+l_2*sin(q_1+q_2)]
            ])
        p_3=Matrix([
            [L_1*cos(q_1)+l_3*cos(q_1+q_2+q_3)],
            [L_1*sin(q_1)+l_3*sin(q_1+q_2+q_3)]
            ])
        p_3f=Matrix([
            [L_1*cos(q_1)+L_3*cos(q_1+q_2+q_3)],
            [L_1*sin(q_1)+L_3*sin(q_1+q_2+q_3)]
            ])
        # ---------------------------------------------------
        
        #substituting in vhcs
        vhc_subs = [ (q_1,phi_1),(q_2,phi_2),(q_3,phi_3) ]
        p_1, p_2, p_3, p_3f = p_1.subs(vhc_subs), p_2.subs(vhc_subs), p_3.subs(vhc_subs), p_3f.subs(vhc_subs)

        #Linear Velocities
        v_1=p_1.diff(t)
        v_2=p_2.diff(t)
        v_3=p_3.diff(t)
        v_3f=p_3f.diff(t)
        
        # derivative of swing-leg foot with repsect to theta
        Dp_3f = p_3f.diff(theta)


        # setting symbolic kin as instance var
        self.p_1, self.p_2, self.p_3, self.p_3f = p_1, p_2, p_3, p_3f
        self.Dp_3f = Dp_3f
        self.v_1, self.v_2, self.v_3, self.v_3f = v_1, v_2, v_3, v_3f
        
        #------------------------------------------------------

    
    def lambdify_kinematics( self ):

        
        theta = self.theta
        theta_d = self.theta_d

        p_1, p_2, p_3, p_3f = self.p_1, self.p_2, self.p_3, self.p_3f
        Dp_3f = self.Dp_3f
        v_1, v_2, v_3, v_3f = self.v_1, self.v_2, self.v_3, self.v_3f

        dum_theta_d = symbols('dum_theta_d')

        v_1 = v_1.subs(theta_d, dum_theta_d) 
        v_2 = v_2.subs(theta_d, dum_theta_d) 
        v_3 = v_3.subs(theta_d, dum_theta_d)
        v_3f = v_3f.subs(theta_d, dum_theta_d)       
        # define types
        mat2array = [{'ImmutableMatrix': np.array}, 'numpy']

        # substituting in physical parameters
        phys_sub = self.phys_params_subs
        p_1, p_2, p_3, p_3f = p_1.subs(phys_sub), p_2.subs(phys_sub), p_3.subs(phys_sub), p_3f.subs(phys_sub)
        Dp_3f = Dp_3f.subs(phys_sub)
        v_1, v_2, v_3, v_3f = v_1.subs(phys_sub), v_2.subs(phys_sub), v_3.subs(phys_sub), v_3f.subs(phys_sub)

        # substituting in vhc-parameterization
        P = self.vhc.get_full_parameter_list()

        self.p1 = lambdify( (theta,P), p_1, modules = mat2array)
        self.p2 = lambdify( (theta,P), p_2, modules = mat2array)
        self.p3 = lambdify( (theta,P), p_3, modules = mat2array)
        self.p3f = lambdify( (theta,P), p_3f, modules = mat2array)
        
        self.Dp3f = lambdify( (theta, P), Dp_3f, modules = mat2array)
 
        self.v1 = lambdify( (theta, dum_theta_d, P), v_1, modules = mat2array)
        self.v2 = lambdify( (theta, dum_theta_d, P), v_2, modules = mat2array)
        self.v3 = lambdify( (theta, dum_theta_d, P), v_3, modules = mat2array)
        self.v3f = lambdify( (theta, dum_theta_d, P), v_3f, modules = mat2array)



    def get_sym_pos( self ):

        return self.p_1, self.p_2, self.p_3,  self.p_3f

    def get_sym_vel( self ):

        return self.v_1, self.v_2, self.v_3,  self.v_3f
    
    def get_sym_Dp_3f( self ):

        return self.Dp_3f
    
    def get_func_Dp_3f( self ):

        return self.Dp3f

    def get_func_pos( self ):

        return self.p1, self.p2, self.p3,  self.p3f

    def get_func_vel( self ):

        return self.v1, self.v2, self.v3,  self.v3f

    def get_func_vel_3f( self ):

        return self.v3f

    
    def load_Model_XML( self, xml_file_name ):
       
        """
        overrides the similiar function in the parent class
        loads the model data from xml and creates new dynamics
 
        """
 
        # getting the root element of the XML document 
        model = ET.parse( xml_file_name ).getroot()
       
        # passing it to the parser
        self.set_data_from_RootElement( model )
        g, m1, m2, m3, L1, L2, L3, l1, l2, l3, J1, J2, J3 = self.get_data()
        
        # saving new physical_parameter_subs as instance variable
        self.phys_params_subs = [
            (symbols('m_1'),m1),(symbols('m_2'),m2),(symbols('m_3'),m3),
            (symbols('L_1'),L1),(symbols('L_2'),L2),(symbols('L_3'),L3),
            (symbols('l_1'),l1),(symbols('l_2'),l2),(symbols('l_3'),l3),
            (symbols('J_1'),J1),(symbols('J_2'),J2),(symbols('J_3'),J3),
            (symbols('g'),g)
            ] 

