import sys
sys.path.append("../q_space_dynamics/")

from biped_dynamics_v1 import Biped_Dynamics

from sympy import *
import numpy as np



class Alpha_Beta_Gamma( object ):


    def __init__( self, convention, model_xml, vhc ):

	self.convention = convention
        
        biped_dynamics = Biped_Dynamics( convention, model_xml )
        
        self.biped_dynamics = biped_dynamics
        self.vhc = vhc

        phi_1, phi_2, phi_3 = vhc.get_sym_vhcs()
        self.derive_abg(phi_1,phi_2,phi_3)



    def derive_abg( self, phi_1, phi_2, phi_3 ):
        
        """
        derives LHS of alpha,beta,gamma - equations
        """

        # symbolic definitions

        t = symbols('t')
        theta = Function('theta')(t)
        theta_d = diff(theta, t)
        theta_dd = diff(theta, t, t)

        phi_1_d = diff(phi_1,t)
        phi_2_d = diff(phi_2,t)
        phi_3_d = diff(phi_3,t)
        phi_1_dd = diff(phi_1,t,t)
        phi_2_dd = diff(phi_2,t,t)
        phi_3_dd = diff(phi_3,t,t)

        # define symbols for generalized coordinates
        q_1=Function('q_1')(t)
        q_2=Function('q_2')(t)
        q_3=Function('q_3')(t)

        #define dot(q_i) and ddot(q_i) to be symbols Dq_i and DDq_i
        q_1_d = diff(q_1,t)
        q_2_d = diff(q_2,t)
        q_3_d = diff(q_3,t)
        q_1_dd = diff(q_1,t,t)
        q_2_dd = diff(q_2,t,t)
        q_3_dd = diff(q_3,t,t)

        # obtain biped_dynamics lhs
        lhs_1, lhs_2, lhs_3 = self.biped_dynamics.get_LHS()

        vhc_subs = [
              (q_1_dd,phi_1_dd),(q_1_d,phi_1_d),(q_1,phi_1),
              (q_2_dd,phi_2_dd),(q_2_d,phi_2_d),(q_2,phi_2),
              (q_3_dd,phi_3_dd),(q_3_d,phi_3_d),(q_3,phi_3)
              ]

        vhc_subs_1 = [ (q_1_dd,phi_1_dd),(q_1_d,phi_1_d),(q_1,phi_1) ]
        vhc_subs_2 = [ (q_2_dd,phi_2_dd),(q_2_d,phi_2_d),(q_2,phi_2) ]
        vhc_subs_3 = [ (q_3_dd,phi_3_dd),(q_3_d,phi_3_d),(q_3,phi_3) ]
     
        
        abg_1 = lhs_1.subs(vhc_subs)
        abg_2 = lhs_2.subs(vhc_subs)
        abg_3 = lhs_3.subs(vhc_subs)
        
        self.abg_1 = abg_1
        self.abg_2 = abg_2
        self.abg_3 = abg_3
        
        abg_1 = abg_1.expand()
        abg_2 = abg_2.expand()
        abg_3 = abg_3.expand()

        abg_1 = collect(abg_1,diff(theta,t,t), exact = True)
        abg_1 = collect(abg_1,diff(theta,t))
       
        abg_2 = collect(abg_2,diff(theta,t,t), exact = True)
        abg_2 = collect(abg_2,diff(theta,t))
         
        abg_3 = collect(abg_3,diff(theta,t,t), exact = True)
        abg_3 = collect(abg_3,diff(theta,t))

        alpha_1 = abg_1.coeff(diff(theta,t,t),1)
        alpha_2 = abg_2.coeff(diff(theta,t,t),1)
        alpha_3 = abg_3.coeff(diff(theta,t,t),1)

        beta_1 = abg_1.coeff(diff(theta,t),2)
        beta_2 = abg_2.coeff(diff(theta,t),2)
        beta_3 = abg_3.coeff(diff(theta,t),2)

        gamma_1 = abg_1 - alpha_1*diff(theta,t,t) - beta_1*diff(theta,t)**2
        gamma_2 = abg_2 - alpha_2*diff(theta,t,t) - beta_2*diff(theta,t)**2
        gamma_3 = abg_3 - alpha_3*diff(theta,t,t) - beta_3*diff(theta,t)**2


        if self.convention == "relative":

            alpha_0 = alpha_1
            beta_0 = beta_1
            gamma_0 = gamma_1

            theta_dd = -(beta_0/alpha_0)*theta_d**2 - (gamma_0/alpha_0)
            
            tau_2 = alpha_2*theta_dd + beta_2*theta_d**2 + gamma_2
            tau_3 = alpha_3*theta_dd + beta_3*theta_d**2 + gamma_3

       
        elif self.convention == "absolute":

            alpha_0 = alpha_1 + alpha_2 + alpha_3
            beta_0 = beta_1 + beta_2 + beta_3
            gamma_0 = gamma_1 + gamma_2 + gamma_3

            theta_dd = -(beta_0/alpha_0)*theta_d**2 - (gamma_0/alpha_0)

            tau_2 = alpha_1*theta_dd + beta_1*theta_d**2 + gamma_1
            tau_3 = alpha_3*theta_dd + beta_3*theta_d**2 + gamma_3


        # storing symbolic expressions as class instance variables
        self.alpha_1, self.alpha_2, self.alpha_3 = alpha_1, alpha_2, alpha_3
        self.beta_1, self.beta_2, self.beta_3 = beta_1, beta_2, beta_3
        self.gamma_1, self.gamma_2, self.gamma_3 = gamma_1, gamma_2, gamma_3
        
        self.alpha_0 = alpha_0
        self.beta_0 = beta_0 
        self.gamma_0 = gamma_0

        self.tau_2 = tau_2
        self.tau_3 = tau_3
        
        # creates a lambdify'd versions of the 
        # alpha,beta,gamma expressions for numerical evaluation 
        
        P = self.vhc.get_full_parameter_list()
        

        self.alpha1 =  lambdify( (theta,P), alpha_1, 'numpy' )
        self.alpha2 =  lambdify( (theta,P), alpha_2, 'numpy' )
        self.alpha3 =  lambdify( (theta,P), alpha_3, 'numpy' )

        self.beta1 = lambdify( (theta,P), beta_1, 'numpy' )
        self.beta2 = lambdify( (theta,P), beta_2, 'numpy' )
        self.beta3 = lambdify( (theta,P), beta_3, 'numpy' )
        
        self.gamma1 =  lambdify( (theta,P), gamma_1, 'numpy' )
        self.gamma2 =  lambdify( (theta,P), gamma_2, 'numpy' )
        self.gamma3 =  lambdify( (theta,P), gamma_3, 'numpy' )

        self.alpha0 = lambdify( (theta,P), alpha_0, 'numpy' )
        self.beta0 = lambdify( (theta,P), beta_0, 'numpy' )
        self.gamma0 = lambdify( (theta,P), gamma_0, 'numpy' )
        
        dum_theta_dot = symbols('dum_theta_dot')
        
        tau_2 = tau_2.subs(theta_d,dum_theta_dot)
        tau_3 = tau_3.subs(theta_d,dum_theta_dot)

        self.tau2 = lambdify( (theta, dum_theta_dot, P), tau_2, 'numpy' )
        self.tau3 = lambdify( (theta, dum_theta_dot, P), tau_3, 'numpy' )



    def create_abg0_sim( self ):

        """
        creates lambdified abg_0(theta)
        substitutes in the current vhcs parameterization
        so that alpha, beta gamma only need theta as arg
        """
        t = symbols('t')
        theta = Function('theta')(t)

        vhc_param_subs = self.vhc.get_full_parameter_dict()
        alpha_0, beta_0, gamma_0 = self.get_sym_abg_0()

        alpha_0_sim = alpha_0.subs(vhc_param_subs)
        beta_0_sim = beta_0.subs(vhc_param_subs)
        gamma_0_sim = gamma_0.subs(vhc_param_subs)

        self.alpha0_sim = lambdify( theta, alpha_0_sim, 'numpy' )
        self.beta0_sim = lambdify( theta, beta_0_sim, 'numpy' )
        self.gamma0_sim = lambdify( theta, gamma_0_sim, 'numpy' )

    
    
    
    def create_tau_sim( self ):
        
        """
        derive torque expressions 
        for use in simulations
        """
        
        tau_2, tau_3 = self.get_sym_tau_2(), self.get_sym_tau_3()

        t= symbols('t')
        theta = Function('theta')(t)
        theta_d = diff(theta, t)

        dum_theta_dot = symbols('dum_theta_dot')
        
        tau_2 = tau_2.subs(theta_d,dum_theta_dot)
        tau_3 = tau_3.subs(theta_d,dum_theta_dot)

        vhc_param_subs = self.vhc.get_full_parameter_dict()

        tau_2 = tau_2.subs(vhc_param_subs)
        tau_3 = tau_3.subs(vhc_param_subs)
        
        self.tau2_sim = lambdify( (theta, dum_theta_dot), tau_2, 'numpy' )
        self.tau3_sim = lambdify( (theta, dum_theta_dot), tau_3, 'numpy' )


    
    def rhs_0( self, theta_theta_d ):

        """
        right hand side of
        theta_dd = rhs
        for zero-dynamics
        """


        theta = theta_theta_d[0]
        theta_d = theta_theta_d[1]
        
        alpha, beta, gamma = self.get_abg0_sim()

        rhs = -(beta(theta)/alpha(theta))*theta_d**2 - (gamma(theta)/alpha(theta)) 
        
        return np.array([theta_d, rhs])
    
    
    def rhs_0_t( self, theta_theta_d, t):

        return self.rhs_0(theta_theta_d)

    
    def get_abg0_sim( self ):

        """
        returns lambdified abg0 for the current 
        vhc parameterization
        """


        return self.alpha0_sim, self.beta0_sim, self.gamma0_sim

    def get_tau_sim( self ):

        """
        returns lambdified torque expressions for the current
        vhc parameterization
        """

        return self.tau2_sim, self.tau3_sim




    def get_vhc( self ):
        
        return self.vhc 

    def get_sym_abg_0( self ):

        return self.alpha_0, self.beta_0, self.gamma_0
    
    def get_sym_abg_1( self ):

        return self.alpha_1, self.beta_1, self.gamma_1

    def get_sym_abg_2( self ):

        return self.alpha_2, self.beta_2, self.gamma_2

    def get_sym_abg_3( self ):

        return self.alpha_3, self.beta_3, self.gamma_3


    def get_sym_tau_2( self ):

        return self.tau_2

    def get_sym_tau_3( self ):

        return self.tau_3
    
    def get_func_abg_1( self ):

        return self.alpha1, self.beta1, self.gamma1
    
    def get_func_abg_2( self ):

        return self.alpha2, self.beta2, self.gamma2

    def get_func_abg_3( self ):

        return self.alpha3, self.beta3, self.gamma3

    def get_func_abg_0( self ):
        
        return self.alpha0, self.beta0, self.gamma0

    def get_func_tau_2( self ):

        return self.tau2

    def get_func_tau_3( self ):

        return self.tau3
    
